#include <stdio.h>
#include <stdlib.h>
#include "getopt.h"
#include <string.h>
#include <ctype.h>

#ifndef min
#define min(a,b) (((a)<(b)) ? (a) : (b))
#endif

#ifndef TRUE
#define TRUE (1==1)
#endif

#ifndef FALSE
#define FALSE (1==0)
#endif

int main(int argc, char **argv)
{
    FILE *fhIn;
    FILE *fhOut;
    char sAsmMneumonic[32];
    char sAsmPrefix[32];
    int type = 0;
    sprintf(sAsmPrefix,"&");
  sprintf(sAsmMneumonic, "defb");
    char c;
    char sLabel[256];
    char *pSourceFilename = NULL;
    char *pDestFilename = NULL;
    sprintf(sLabel,"datablock");
  char sLinePrefix[256];
    //const char *sPrefix;
    char *sPrefix, *sCodeStart, *sCodeEnd, *sLineEndContinue;


	if (argc==1)
	{
		printf("datamaker will create data for C or ASM code.\n");
		printf("Usage: datamaker [arguments] <input filename> <output filename>\n\n");
		printf("-t              - Type\n");
		printf("                  0 = ASM (default)\n");
		printf("                  1 = C\n");
		printf("-l              - Label\n");
		printf("-p              - Asm prefix (& default)\n");
		printf("-m              - Asm mneumonic (defb default)\n");
	}
	else
	{
        do
        {
            c = getopt(argc, argv,"p:t:l:");

            switch (c)
            {
                case 'l':
                {
                  int i;

                  /* limit length */
                  strncpy(sLabel, optarg, sizeof(sLabel));
                  /* ensure null */
                  sLabel[sizeof(sLabel)-1]='\0';
                  /* if no length, generate a label name */
                  if (strlen(sLabel)==0)
                  {
                      sprintf(sLabel,"code");
                  }

                    /* substitute */
                  if (isdigit(sLabel[0]))
                  {
                    sLabel[0]='_';
                  }
                  
                  /* remove invalid chars */
                  for (i=0; i<strlen(sLabel); i++)
                  {
                    char ch;
                    
                      ch = sLabel[i];
                    
                    /* digit and alpha chars allowed and underline */
                    if (
                        (!isdigit(ch)) && 
                      (!isalpha(ch)) &&
                      (ch!='_'))
                    {
                      sLabel[i] = '_';
                    }
                                        
                    
                  }
                }
                break;

                case 't':
                {
                    type = atoi(optarg);

                }
                break;
    case 'm':
    {
      strcpy(sAsmMneumonic, optarg);
    }
    break;
    
				case 'p':
                {
                    strcpy(sAsmPrefix,optarg);
                }
                break;

                default:
                {
                }
                break;
            }

        }
        while (c!=-1);

        if ((argc-optind)==0)
        {
            printf("No source file or sLinePrefixdestination file have been specified!\n");
            exit(1);
        }

        if ((argc-optind)==1)
        {
            printf("No destination file has been specified\n");
            exit(1);
        }

        pSourceFilename = argv[optind];
        pDestFilename = argv[optind+1];

        if (type==1)
        {
            /* this is C */
          sprintf(sLinePrefix,"\t");
            sPrefix = "0x";				// prefix numbers
            sCodeStart = "const unsigned char %s[%d] =\n{\n";		// prefix code
            sCodeEnd = "\n}; /* %s */\n";	// end code
            sLineEndContinue = ",";
        }
        else
        {
            /* this is asm */
          sprintf(sLinePrefix, "%s ", sAsmMneumonic);
            sPrefix = sAsmPrefix;
            sCodeStart = ".%s\n ;;%d bytes\n";
            sCodeEnd = ";; %s\n";	// end code
            sLineEndContinue = "";
        }

        /* input, output, flags for asm or not */

        fhOut= fopen(pDestFilename,"w");
        if (fhOut!=NULL)
        {
            fhIn = fopen(pSourceFilename,"rb");
            if (fhIn!=NULL)
            {
                unsigned long Length;
                unsigned char *pData;

                fseek(fhIn,0,SEEK_END);
                Length = ftell(fhIn);
                fseek(fhIn,0,SEEK_SET);

                pData = (unsigned char *)malloc(Length);
                if (pData!=NULL)
                {
                    unsigned long LengthRemaining;
                    unsigned long Position;
                    int LineLength;
                  //  int StartOfLine = TRUE;

                    if (fread(pData,1,Length,fhIn)==Length)
		    {

			    /* code prefix */
			    fprintf(fhOut, sCodeStart,  sLabel, Length);

			    Position = 0;
			    LengthRemaining = Length;

			    while (LengthRemaining!=0)
			    {
				int b;

				LineLength = min(16, LengthRemaining);

				/* prefix for each line */
				fprintf(fhOut,"%s",sLinePrefix);

				for (b=0; b<LineLength; b++)
				{
				    /* number */
				    fprintf(fhOut,"%s0%02x",sPrefix,pData[Position]);
				    Position++;

				    /* number seperator */
				    if (b!=(LineLength-1))
					fprintf(fhOut,",");

				}

				LengthRemaining-=LineLength;

				/* continuation string */
				if (LengthRemaining!=0)
				    fprintf(fhOut,"%s",sLineEndContinue);

				/* end of current line */
				fprintf(fhOut,"\n");

			    }

			    /* code end */
			    fprintf(fhOut, sCodeEnd, sLabel);
		    }


                    free(pData);
                }



                fclose(fhIn);
            }
            else
            {
                fprintf(stdout,"Failed to open \"%s\"\n", pSourceFilename);
            }

            fclose(fhOut);
        }
        else
        {
            fprintf(stdout,"Failed to open \"%s\"\n", pDestFilename);
        }
    }
    return 0;
}
